#!/bin/bash

############################################################################################
# COPYRIGHT LICENSE:  This utility is owned by IBM and is provided in source code form. 
# You may copy and use (but not modify)  the utility  without payment to IBM for the 
# purposes of helping you to migrate your data from an IBM Workload Deployer product to
# another IBM product. Notwithstanding anything to the contrary,  IBM PROVIDES THE UTILITY
# ON AN "AS IS" BASIS AND IBM DISCLAIMS ALL WARRANTIES, EXPRESS OR IMPLIED, INCLUDING,
# BUT NOT LIMITED TO, ANY IMPLIED WARRANTIES OR CONDITIONS OF MERCHANTABILITY, SATISFACTORY
# QUALITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE, AND ANY WARRANTY OR CONDITION OF
# NON-INFRINGEMENT. IBM SHALL NOT BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL
# OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE OR OPERATION OF THE UTLITY.  
# IBM MAY PROVIDE LIMITED SUPPORTFOR THE UTLITY, BUT IBM HAS NO OBLIGATION TO PROVIDE
# MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS OR MODIFICATIONS TO THE UTLITITY.
###########################################################################################

readParameter(){
	PARAMETER_NAME=$1
	PARAMETER_VALUE=""
	if [ -z "$PARAMETER_NAME" ] ; then
			echo "No parameter name specified. Exiting"
			exit 2
	fi
	stty -echo
	read -p "Value: " PARAMETER_VALUE
	stty echo
	printf '\n'	
	printf -v "$PARAMETER_NAME" "$PARAMETER_VALUE"
	export "$PARAMETER_NAME"
	return 0
}

verifyIWD3107Credentials(){
	echo "Verifying IWD 3107 credentials..."	
	if [ -z "$IWD_3107_HOSTNAME" ] || [ -z "$IWD_3107_USER" ] ; then
		echo "IWD 3107 parameters aren't set. Please provide: iwd_3107_hostname and iwd_3107_admin username in property file $DATA_FILE"
		return 1
	fi
	if  [ -z "$IWD_3107_PASSWORD" ] ; then
		echo "IWD 3107 admin password not specified."
		echo "Please enter the password below"
		readParameter IWD_3107_PASSWORD
	fi
}

verifySCOCredentials(){
	echo "Verifying SCO credentials..."
	if [ -z "$SCO_HOSTNAME" ] || [ -z "$SCO_USER" ] ; then
		echo "SCO parameters aren't set. Please provide: sco_hostname and sco_admin username in property file $DATA_FILE"
		return 1
	fi
	if [ -z "$SCO_PASSWORD" ] ; then
		echo "SCO admin password not specified."
		echo "Please enter the password below"
		readParameter SCO_PASSWORD
	fi
}

verifyRequiredParams(){
	verifyIwdCliDirectory
	verifyMigrationDirectory	
}

verifyIwdCliDirectory(){
	if [ -z "$IWD_CLI" ] ; then
		echo "IWD CLI directory is not specified. Please set IWD_CLI variable in  property file $DATA_FILE"
		echo ""
		exit 1
	fi
	
	if [ ! -d "$IWD_CLI" ] ; then
		echo "IWD CLI directory doesn't exists. Please verify the location of IWD CLI in property file $DATA_FILE"
		echo ""
	    exit 1
	fi
}

verifyMigrationDirectory(){
	if [ -z "$MIGRATION_DEST" ] ; then
		echo "Migration directory is not specified. Please set MIGRATION_DEST variable in property file $DATA_FILE"
		echo ""
		exit 1
	fi	
}

