#!/bin/bash

############################################################################################
# COPYRIGHT LICENSE:  This utility is owned by IBM and is provided in source code form. 
# You may copy and use (but not modify)  the utility  without payment to IBM for the 
# purposes of helping you to migrate your data from an IBM Workload Deployer product to
# another IBM product. Notwithstanding anything to the contrary,  IBM PROVIDES THE UTILITY
# ON AN "AS IS" BASIS AND IBM DISCLAIMS ALL WARRANTIES, EXPRESS OR IMPLIED, INCLUDING,
# BUT NOT LIMITED TO, ANY IMPLIED WARRANTIES OR CONDITIONS OF MERCHANTABILITY, SATISFACTORY
# QUALITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE, AND ANY WARRANTY OR CONDITION OF
# NON-INFRINGEMENT. IBM SHALL NOT BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL
# OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE OR OPERATION OF THE UTLITY.  
# IBM MAY PROVIDE LIMITED SUPPORTFOR THE UTLITY, BUT IBM HAS NO OBLIGATION TO PROVIDE
# MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS OR MODIFICATIONS TO THE UTLITITY.
###########################################################################################

OPENRC_FILE="/root/openrc"
if [ ! -f $OPENRC_FILE ] ; then
	echo "Openrc file $OPENRC_FILE doesn't exists. Script cannot continue"
	exit 2
fi 
source $OPENRC_FILE

CURRENT_DIR=$(dirname "$0")
DATA_FILE="$CURRENT_DIR/configureOpenstack.txt"

timestamp=$(date +"%y%m%d_%H%M%S")
LOG_FILE="$CURRENT_DIR/configureOpenstack_${timestamp}.log"

readParameter(){
	PARAMETER_NAME=$1
	PARAMETER_VALUE=""
	if [ -z "$PARAMETER_NAME" ] ; then
			echo "No parameter name specified. Exiting"
			exit 2
	fi
	stty -echo
	read -p "Value: " PARAMETER_VALUE
	stty echo
	printf '\n'	
	printf -v "$PARAMETER_NAME" "$PARAMETER_VALUE"
	export "$PARAMETER_NAME"
	return 0
}

while [ $# -gt 0 ]
do
	if [ -z "$2" ] ; then
    	echo "No argument supplied for option $1. Exiting the script."
    	exit 2
	fi
    case $1 in
		-f*|--file*)
		DATA_FILE=$2
		;;                
		*)
		echo "Unknow option: $1. Exiting the script."
		exit 2
		;;
    esac
    shift 2
done
if [ ! -e "$DATA_FILE" ] ; then
	echo "Property file $DATA_FILE doesn't exist. Please provide valid path by specyfing -f option"
	exit 1
fi
source $DATA_FILE

echo "Configuring Openstack with VMWare..." 2>&1 | tee -a $LOG_FILE
echo "Logging output to a file $LOG_FILE"
echo "Using property file: $DATA_FILE"  2>&1 | tee -a $LOG_FILE
echo ""

if  [ -z "$VCENTER_IP" ] || [ -z "$VCENTER_USER" ]  || [ -z "$VCENTER_NETWORK_NAME" ] || [ -z "$VMWARE_CLOUD_NAME" ] ; then
	echo "Configuration parameters aren't set. Please update data file $DATA_FILE with required attributes for Openstack configuration" 2>&1 | tee -a $LOG_FILE
	exit 1
fi	
if [ -z "$VCENTER_PASSWORD" ] ; then
	echo "VCenter administrator password was not specified. Please enter the password:"
	readParameter VCENTER_PASSWORD
fi

OPENSTACK_HOSTNAME=`hostname -f`
NETWORK_ID=`nova network-list | sed -n 4p | cut -d '|' -f 2`
if [ -z "$NETWORK_ID" ] ; then
	echo "Error while obtaining network id" 2>&1 | tee -a $LOG_FILE
	echo "Cannot continue. Exiting" 2>&1 | tee -a $LOG_FILE
	exit 1
fi

echo "Creating new network at Openstack with name $VMWARE_CLOUD_NAME ..." 2>&1 | tee -a $LOG_FILE
/opt/ibm/openstack/iaas/smartcloud/bin/nova-cloud-create $OPENSTACK_HOSTNAME "$VMWARE_CLOUD_NAME" $VCENTER_IP $VCENTER_USER $VCENTER_PASSWORD VMware 2>&1 | tee -a $LOG_FILE 
if [ $? -eq 0 ] ; then
	echo "Network created!" 2>&1 | tee -a $LOG_FILE
	echo "" 2>&1 | tee -a $LOG_FILE
	echo "Matching new network with VMWare network \"$VCENTER_NETWORK_NAME\" ..." 2>&1 | tee -a $LOG_FILE
	/opt/ibm/openstack/iaas/smartcloud/bin/nova-netext-match $VMWARE_CLOUD_NAME "$VCENTER_NETWORK_NAME" $NETWORK_ID 2>&1 | tee -a $LOG_FILE
	if [ $? -eq 0 ]; then
		echo "Networks matched!" 2>&1 | tee -a $LOG_FILE
		echo "" 2>&1 | tee -a $LOG_FILE
	else
		echo "Error while matching networks. Configuration hasn't been completed" 2>&1 | tee -a $LOG_FILE
		exit 1
	fi
else
	echo "Error while creating new network. Configuration hasn't been completed" 2>&1 | tee -a $LOG_FILE
	exit 1
fi

echo "Configuration completed" 2>&1 | tee -a $LOG_FILE
echo ""
exit 0