#!/bin/bash

############################################################################################
# COPYRIGHT LICENSE:  This utility is owned by IBM and is provided in source code form. 
# You may copy and use (but not modify)  the utility  without payment to IBM for the 
# purposes of helping you to migrate your data from an IBM Workload Deployer product to
# another IBM product. Notwithstanding anything to the contrary,  IBM PROVIDES THE UTILITY
# ON AN "AS IS" BASIS AND IBM DISCLAIMS ALL WARRANTIES, EXPRESS OR IMPLIED, INCLUDING,
# BUT NOT LIMITED TO, ANY IMPLIED WARRANTIES OR CONDITIONS OF MERCHANTABILITY, SATISFACTORY
# QUALITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE, AND ANY WARRANTY OR CONDITION OF
# NON-INFRINGEMENT. IBM SHALL NOT BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL
# OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE OR OPERATION OF THE UTLITY.  
# IBM MAY PROVIDE LIMITED SUPPORTFOR THE UTLITY, BUT IBM HAS NO OBLIGATION TO PROVIDE
# MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS OR MODIFICATIONS TO THE UTLITITY.
###########################################################################################

INSTALL_DIR=$(dirname "$0")
DATA_FILE="$INSTALL_DIR/migration_data.txt"
MIGRATION_MODULE="$INSTALL_DIR/scomigrator.tar.gz"

COMMONS="$INSTALL_DIR/commons.sh"
if [ ! -f $COMMONS ] ; then
	echo "Wrong path to script commons.sh. Path: $COMMONS. Tool cannot continue"
	exit 2
fi
source $COMMONS

while [ $# -gt 0 ]
do
	if [ -z "$2" ] ; then
    	echo "No argument supplied for option $1. Exiting the script."
    	exit 2
	fi
	case $1 in
		-f*|--file*)
		DATA_FILE=$2
		;;
		-m*|--mode*)
		MODE=$2
		;;
		-a*|--artifacts*)
		ARTIFACTS=$2
		;;
		*)
		echo "Unknow option: $1. Exiting the script."
		exit 2
		;;
	esac
	shift 2
done

echo "--------------------------------------------"
echo "Install tool started"
echo "--------------------------------------------"

if [ ! -f "$DATA_FILE" ] ; then
	echo "Property file $DATA_FILE doesn't exist. Please provide valid path by specyfing -f option"
	echo ""
	exit 1
fi

source $DATA_FILE
verifyRequiredParams

echo "Installing migration tool ..." ; echo ""

VERIFICATION_SUCCESSFUL=0
verifyIWD3107Credentials
if [ $? -eq 0 ] ; then
	echo "Connecting to IWD server ..."
	$IWD_CLI/bin/deployer -h $IWD_3107_HOSTNAME -u $IWD_3107_USER -p $IWD_3107_PASSWORD -a -c "print \"OK\""
	if [ $? -ne 0 ] ; then
		echo "Bad credentials, cannot continue"
		exit 1
	fi
	VERIFICATION_SUCCESSFUL=1
fi
echo""

verifySCOCredentials
if [ $? -eq 0 ] ; then
	echo "Connecting to SCO server ..."
	$IWD_CLI/bin/deployer -h $SCO_HOSTNAME -u $SCO_USER -p $SCO_PASSWORD -a -c "print \"OK\""
	if [ $? -ne 0 ] ; then
		echo "Bad credentials, cannot continue"
		exit 1
	fi
	VERIFICATION_SUCCESSFUL=1
fi
echo ""


if [ $VERIFICATION_SUCCESSFUL -eq 1 ] ; then
	echo "Installing migration module ..."
	for dir in `ls -d ${IWD_CLI}/lib/*/` ; do
		tar -zxf $MIGRATION_MODULE -C $dir
		if [ $? -eq 0 ] ; then
			echo "Installed to $dir successfully"
		else
			echo "Error while extracting migration module to $dir"
			exit 1
		fi
	done
	echo ""
	echo "Installation completed"
	echo ""	
	exit 0
else
	echo "Tool wasn't able to connect neither to IWD 3107 nor SCO. Exiting"
	exit 2
fi