'''
COPYRIGHT LICENSE:  This utility is owned by IBM and is provided in source code form. 
You may copy and use (but not modify)  the utility  without payment to IBM for the 
purposes of helping you to migrate your data from an IBM Workload Deployer product to
another IBM product. Notwithstanding anything to the contrary,  IBM PROVIDES THE UTILITY
ON AN "AS IS" BASIS AND IBM DISCLAIMS ALL WARRANTIES, EXPRESS OR IMPLIED, INCLUDING,
BUT NOT LIMITED TO, ANY IMPLIED WARRANTIES OR CONDITIONS OF MERCHANTABILITY, SATISFACTORY
QUALITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE, AND ANY WARRANTY OR CONDITION OF
NON-INFRINGEMENT. IBM SHALL NOT BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL
OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE OR OPERATION OF THE UTLITY.  
IBM MAY PROVIDE LIMITED SUPPORTFOR THE UTLITY, BUT IBM HAS NO OBLIGATION TO PROVIDE
MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS OR MODIFICATIONS TO THE UTLITITY.
'''

import codecs
import getopt
import sys
import os
import subprocess
import re
import base64
import httplib
import time
import shutil
from xml.dom import minidom
from deployer import ziputils
from deployer.messages import message
from deployer import facetutils, http, patterntopython, prettify, utils, validators, patternloader, patternutils
from scomigrator import *
from scomigrator import db2, constants

from java.io import File

def prepareArtifactsForMigration(artifacts, artifactList):
	if artifacts == "all" :
		for type in artifactTypes:
			artifactList[type] = 1
	else:	
		for type in artifactTypes:
			for artifact in artifacts:
				if type == artifact:
					artifactList[type] = 1
	print "Artifacts to migrate: %s" % (str(artifacts))	

def migrateArtifacts(dest,artifactList,mode, options):	
	if mode == "export" and checkDiskSpace(dest,artifactList,options):
#	if mode == "export":
		exportArtifacts(dest, artifactList, options)
	elif mode == "import":
		importArtifacts(dest, artifactList, options)

def exportArtifacts(dest, artifactList, options):
	print "Exporting artifacts to %s" % (dest)
	print "--------------------------------------------\n"
	isError = 0
	timestamp = time.strftime("%m-%d_%H%M%S", time.localtime())
	logFile = "export_" + timestamp + ".log"
	tmpStdout = sys.stdout
	tmpStderr = sys.stderr
	try:
		f = initLog(logFile)
		if artifactList['user']:
			users.exportUsers(dest)
			groups.exportUserGroups(dest)
		if artifactList['script']:		
			script_packages.exportScriptPackages(dest)		
		if artifactList['addon']:
			add_ons.exportAddOns(dest)		
		if artifactList['virtimg']:
			virtual_images.exportVirtualImages(dest)
		if artifactList['vsys']:
			virtual_system_patterns.exportVirtualSystemPatterns(dest)
		if artifactList['ptype']:
			pattern_types.exportPatternTypes(dest)
		if artifactList['efix']:
			# Currently only export, as SCO 2.3 doesnt support efixes.
			emergency_fixes.exportEmergencyFixes(dest, options)		
		if artifactList['vapp']:
			virtual_application_patterns.exportVirtualApplicationPatterns(dest)	
		print "Export of artifacts completed"
	except Exception, err:
		migration_exceptions.printGeneralException(err, str(err))
		isError = 1
	finally:
		f.close()
		sys.stdout = tmpStdout
		sys.stderr = tmpStderr
	print "--------------------------------------------"
	if isError:
		print "Export of artifact failed\n"
	else:
		print "Export of artifacts completed\n"
	
def importArtifacts(dest, artifactList, options):
	print "Importing artifacts from %s\n" % (dest)
	print "--------------------------------------------\n"
	isError = 0
	timestamp = time.strftime("%m-%d_%H%M%S", time.localtime())
	logFile = "import_" + timestamp + ".log"
	tmpStdout = sys.stdout
	tmpStderr = sys.stderr
	try:
		f = initLog(logFile)	
		if artifactList['user'] == 1:
			importArtifact('user',options)
		if artifactList['script'] == 1 :
			importArtifact ('script',options)
		if artifactList['addon'] == 1 :
			importArtifact ('addon',options)
		if artifactList['virtimg'] == 1 :
			importArtifact ('virtimg',options)
		if artifactList['vsys'] == 1 :
			importArtifact ('vsys',options)
		if artifactList['ptype'] == 1 :
			importArtifact ('ptype',options)
		if artifactList['vapp'] == 1 :
			importArtifact ('vapp',options)	
		sqlFile = "migration_" + timestamp+ ".sql"
		db2.createMigrationSql(os.path.join(dest, sqlFile))
		print "Import of artifacts completed"
	except Exception, err:
		migration_exceptions.printGeneralException(err, str(err))
		isError = 1
	finally:
		f.close()
		sys.stdout = tmpStdout
		sys.stderr = tmpStderr
	print "--------------------------------------------"
	if isError:
		print "Import of artifact failed\n"
	else:
		print "Import of artifacts completed\n"

def importArtifact(artifact, options=None):
	if artifact == 'user':		
		authObj = createAuthorizationObject(options['sco_user'], options['sco_password'],options['gateway_hostname'], options['gateway_port'])
		if authObj is None :
			print "Migration tool wasn't able to continue import of Users due to problem with authentication"
			return 1
		envIds = prepareDefaultDomainAndProject(authObj, constants.SCO_DEFAULT_DOMAIN, constants.SCO_DEFAULT_PROJECT)
		if envIds is None:
			print "Migration tool wasn't able to continue import of Users due to problem with creation of default domain or project"
			return 1
		re = users.importUsers(authObj, envIds,dest)
		re = groups.importUserGroups(authObj, envIds, dest)
	elif artifact == 'script':		
		re = script_packages.importScriptPackages(dest,options)		
	elif artifact == 'addon' :
		re = add_ons.importAddOns(dest,options)		
	elif artifact == 'virtimg':
		re = virtual_images.importVirtualImages(dest,options)
#		if re == 0 and options != None and options['check_out'] == "yes" :
#			virtual_images.checkOutVirtualImages(options['scp_hostname'], options['vmware_repository'], options['vmware_datastore'], options['sco_user'], options['sco_password'])				
	elif artifact == 'vsys':
		re = virtual_system_patterns.importVirtualSystemPatterns(dest,options)
	elif artifact == 'ptype':
		re = pattern_types.importPatternTypes(dest)	
	elif artifact == 'vapp':
		re = virtual_application_patterns.importVirtualApplicationPatterns(dest,options)
	
	if re != 0:
		print "Error while importing artifacts. Import stopped !"
		sys.exit(2)

def checkDiskSpace(dest, artifactList, options):
	freeSpace = calculateFreSpace(dest)
	print 'Free space : %d MB' % (freeSpace)
	if artifactList['virtimg'] == 1 :
		requiredSpace = calculateRequiredSpace(options['iwd_3107_hostname'],constants.SCO_HTTPS_PORT,options['iwd_3107_user'],options['iwd_3107_password'] )
		print 'Minimum required space for virtual images: %d MB' % (requiredSpace)
		if requiredSpace >= freeSpace:
			print 'Required space is larger than available on the disk. Exiting\n'
			return False
	return True
		
def calculateFreSpace(dest):
	path = dest
	if not os.path.exists(path):
		patternloader._createDir(path)
	
	try:
		diskPartition = File(dest)	
		freePartitionSpace = diskPartition.getFreeSpace();
		return freePartitionSpace / 1024 / 1024
	except Exception, err:
		return -1
		
def calculateRequiredSpace(host, port, username, password):
	print "Calculating required space ..."
	url = "/resources/virtualImages/IMAGE_ID/viovf"
	totalSize = long(0)
	for image in deployer.virtualimages:
		id = str(image.id)		
		uri = re.sub("IMAGE_ID",id,url)
		resp = rest.iwdRestRequest(host,port, username, password, "GET", uri, None)
		if resp['status'] == rest.STATUS_GET_SUCCESS :
			ovf = patternloader._parseJSON(resp['response'])			
			if ovf != None and ovf[0]['ovfXML'] != None:
				imgSize = long(0)
				xmldoc = minidom.parseString(ovf[0]['ovfXML'])
				fileList = xmldoc.getElementsByTagName('File')
				if len(fileList) == 0:
					fileList = xmldoc.getElementsByTagName('ovf:File')
					if len(fileList) == 0:
						print "\tUnable to get size of the image %s" % (image.name)
						continue				
				for file in fileList:						
					if file.attributes.has_key('size'):
						size = long(file.attributes['size'].value)
					elif file.attributes.has_key('ovf:size'):
						size = long(file.attributes['ovf:size'].value)
					if size :
						imgSize = imgSize + size
				if imgSize > 0 :
					print "\tImage size for %s is %d MB" %(image.name, (imgSize /1024/1024))
					totalSize = totalSize + imgSize
				else:
					print "\tUnable to get size of the image %s" % (image.name)
							
		else:
			print '\tUnable to get size of the image %s' % (image.name)
	return totalSize / 1024 / 1024
		
def createDefaultDomain(authObj, domainName):
	print "Creating default domain for migration purpose: %s" % (constants.SCO_DEFAULT_DOMAIN)
	defaultDomain = domains.getDomain(authObj,constants.SCO_DEFAULT_DOMAIN)
	if defaultDomain is None :		
		domainRestObj = { "domain": {
			"description": "Default domain for migration",
			"enabled": "true",
			"name": domainName
		}}
		defaultDomain = domains.createDomain(authObj, domainRestObj)
		if defaultDomain is None:
			print "Unable to create a migration domain. Exiting"
			return None
		else:
			print "Default domain created"
	else:
		print "Default domain already exists."
	return defaultDomain
	
def createDefaultProject(authObj, projectName, domainId):
	print "Creating default project for migration purpose: %s" % (constants.SCO_DEFAULT_PROJECT)
	defaultProject = projects.getProject(authObj,constants.SCO_DEFAULT_PROJECT)
	if defaultProject is None :		
		projectRestObj = {"project": {
			"description": "Default project for migration",
			"domain_id": domainId,
			"enabled": "true",
			"name": projectName
		}}
		defaultProject = projects.createProject(authObj, projectRestObj)
		if defaultProject is None:
			print "Unable to create a migration project. Exiting"
			return None
		else:
			print "Default project created"
	else:
		print "Default project already exists."
	return defaultProject

def createAuthorizationObject(adminUser,adminPassword,gatewayHost,gatewayPort):	
	if not rest.validateGatewayParams(gatewayHost,gatewayPort):
		print "Tool wasn't able to connect to Gateway. Please verify hostname and port for Gateway."
		raise migration_exceptions.RestConnectionException("Error while testing connection with specified hostname and port")
	endpointId = rest.getKeystoneEndpointId(gatewayHost, gatewayPort)
	if endpointId is None :
		print "Unable to get Iaas Gateway endpoint id."
		return None	
	token = rest.getKeystoneAuthenticationToken(adminUser,adminPassword,endpointId,gatewayHost,gatewayPort)
	if token is None :
		print "Unable to get authentication token. Please verify Gateway parameters."
		return None
	authObj = { 
		"authToken" : token, 
		"endpointId" : endpointId, 
		"gatewayHost" : gatewayHost, 
		"gatewayPort":gatewayPort
	}
	return authObj
	
def prepareDefaultDomainAndProject(authObj, domainName, projectName):
	defaultDomain = createDefaultDomain(authObj,domainName)	
	if defaultDomain is not None:		
		defaultProject = createDefaultProject(authObj, projectName, defaultDomain['id'])
		if defaultProject is not None:
			return { "domain_id" : defaultDomain['id'], "project_id" : defaultProject['id'] }
		else:
			return None
	else:
		return None
	
def initLog(logFile):
	currentDir = os.path.dirname(os.path.realpath(sys.argv[0]))
	logDir = currentDir + "/logs"
	if not os.path.exists(logDir):
		os.makedirs(logDir)
	logFileFqp = logDir + "/" + logFile  
	print "Writing output to : %s\n" % (logFileFqp)
	f = open(logFileFqp,'w')
	sys.stdout = f
	sys.stderr = sys.stdout
	return f
	
dest = None
mode = None
artifacts = None
options = None
migrationModes = ['import','export']
artifactTypes = ['user','script', 'addon', 'virtimg', 'vsys', 'ptype', 'vapp']
artifactList = {'user':0,'script':0, 'addon':0, 'virtimg':0, 'vsys':0, 'ptype':0, 'efix':0, 'vapp':0}	

try:
    (params, args) = getopt.getopt(sys.argv[1:], 'm:d:a', ['mode=', 'data=', 'artifacts='])
except getopt.GetoptError:
    help()

for param in params:
	if param[0] == '-d' or param[0] == '--data':
		options = patternloader._parseJSON(param[1])
		dest = options['dest']
	elif param[0] == '-m' or param[0] == '--mode':        
		mode = param[1]
		if not mode in migrationModes :
			print "Invalid mode. Please choose export or import mode"
			sys.exit(1)
	elif param[0] == '--artifacts':
		artifacts = param[1]
		if artifacts != 'all' :
			artifacts = artifacts.split(',')
			if len(artifacts):
				for a in artifacts:
					if not a in artifactTypes:
						print "Invalid artifact type: %s. Please provide valid parameter from list %s" %(a, str(artifactTypes))
						sys.exit(1)
			else:
				print 'Empty aritfacts list. Please provide valid parameters from list %s' %(str(artifactTypes))
				sys.exit(1)
		
if dest == None or dest == "" :
	print "Migration directory was not specified. Exiting"
	sys.exit(1)
if artifacts == None:
	print "Artifacts for migration was not specified. Exiting"
	sys.exit(1)


prepareArtifactsForMigration(artifacts, artifactList)
migrateArtifacts(dest,artifactList,mode, options)


		
	