#!/bin/bash

############################################################################################
# COPYRIGHT LICENSE:  This utility is owned by IBM and is provided in source code form. 
# You may copy and use (but not modify)  the utility  without payment to IBM for the 
# purposes of helping you to migrate your data from an IBM Workload Deployer product to
# another IBM product. Notwithstanding anything to the contrary,  IBM PROVIDES THE UTILITY
# ON AN "AS IS" BASIS AND IBM DISCLAIMS ALL WARRANTIES, EXPRESS OR IMPLIED, INCLUDING,
# BUT NOT LIMITED TO, ANY IMPLIED WARRANTIES OR CONDITIONS OF MERCHANTABILITY, SATISFACTORY
# QUALITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE, AND ANY WARRANTY OR CONDITION OF
# NON-INFRINGEMENT. IBM SHALL NOT BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL
# OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE OR OPERATION OF THE UTLITY.  
# IBM MAY PROVIDE LIMITED SUPPORTFOR THE UTLITY, BUT IBM HAS NO OBLIGATION TO PROVIDE
# MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS OR MODIFICATIONS TO THE UTLITITY.
###########################################################################################

INSTALL_DIR=$(dirname "$0")
DATA_FILE="$INSTALL_DIR/migration_data.txt"
MIGRATION_FILE="$INSTALL_DIR/migration.py"
MODE=""
ARTIFACTS=""

COMMONS="$INSTALL_DIR/commons.sh"
if [ ! -f $COMMONS ] ; then
	echo "Wrong path to script commons.sh. Path: $COMMONS. Tool cannot continue"
	exit 2
fi
source $COMMONS

while [ $# -gt 0 ]
do
	if [ -z "$2" ] ; then
    	echo "No argument supplied for option $1. Exiting the script."
    	exit 2
	fi
	case $1 in
		-f*|--file*)
		DATA_FILE=$2
		;;
		-m*|--mode*)
		MODE=$2
		;;
		-a*|--artifacts*)
		ARTIFACTS=$2
		;;
		*)
		echo "Unknow option: $1. Exiting the script."
		exit 2
		;;
	esac
	shift 2
done

echo "--------------------------------------------"
echo "Migration tool started"
echo "--------------------------------------------"

if [ ! -e "$DATA_FILE" ] ; then
        echo "Property file $DATA_FILE doesn't exist. Please provide valid path by specyfing -f option"
        echo ""
        exit
fi

# If artifact list is not specified, tool set -a option to "all"
if [ "$ARTIFACTS" == "" ] ; then
	ARTIFACTS="all"
fi

source $DATA_FILE

verifyRequiredParams

OPTIONS="\"dest\":\"$MIGRATION_DEST\""
if [ "$MODE" == "export" ] ; then
	verifyIWD3107Credentials
	if [ $? -eq 0 ] ; then
		OPTIONS="$OPTIONS , \"iwd_3107_hostname\":\"$IWD_3107_HOSTNAME\",
		\"iwd_3107_user\":\"$IWD_3107_USER\",\"iwd_3107_password\":\"$IWD_3107_PASSWORD\""
		OPTIONS="{$OPTIONS}"
		$IWD_CLI/bin/deployer -h $IWD_3107_HOSTNAME -u $IWD_3107_USER -p $IWD_3107_PASSWORD -f $MIGRATION_FILE -m $MODE --artifacts "$ARTIFACTS" -d "$OPTIONS"
	fi
elif [ "$MODE" == "import" ] ; then
	verifySCOCredentials
	if [ $? -eq 0 ] ; then
		OPTIONS="$OPTIONS ,\"scp_hostname\":\"$SCO_HOSTNAME\",
		\"sco_user\":\"$SCO_USER\", \"sco_password\":\"$SCO_PASSWORD\",
		\"gateway_hostname\":\"$GATEWAY_HOSTNAME\",\"gateway_port\":$GATEWAY_PORT"	
	#	OPTIONS="$OPTIONS , \"db2_hostname\":\"$DB2_HOSTNAME\", \"db2_user\":\"$DB2_USER\""
		OPTIONS="{$OPTIONS}"		
		$IWD_CLI/bin/deployer -h $SCO_HOSTNAME -u $SCO_USER -p $SCO_PASSWORD -f $MIGRATION_FILE -m $MODE --artifacts "$ARTIFACTS" -d "$OPTIONS"
	fi
else
	echo "Invalid mode specified. Available modes: export, import."
	exit 1
fi		


exit 0