'''
COPYRIGHT LICENSE:  This utility is owned by IBM and is provided in source code form. 
You may copy and use (but not modify)  the utility  without payment to IBM for the 
purposes of helping you to migrate your data from an IBM Workload Deployer product to
another IBM product. Notwithstanding anything to the contrary,  IBM PROVIDES THE UTILITY
ON AN "AS IS" BASIS AND IBM DISCLAIMS ALL WARRANTIES, EXPRESS OR IMPLIED, INCLUDING,
BUT NOT LIMITED TO, ANY IMPLIED WARRANTIES OR CONDITIONS OF MERCHANTABILITY, SATISFACTORY
QUALITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE, AND ANY WARRANTY OR CONDITION OF
NON-INFRINGEMENT. IBM SHALL NOT BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL
OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE OR OPERATION OF THE UTLITY.  
IBM MAY PROVIDE LIMITED SUPPORTFOR THE UTLITY, BUT IBM HAS NO OBLIGATION TO PROVIDE
MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS OR MODIFICATIONS TO THE UTLITITY.
'''

import codecs
import os
import deployer
import traceback
import zipfile, tarfile
import subprocess
import re
from deployer import patternloader
from scomigrator import acls,rest, constants, db2, migration_exceptions

JSON_FILE = 'addon.json'
RESOURCE_TYPE="addons"

def _recordAddonAttributes(addon):
	addonObj = {patternloader.NAME: addon.name, patternloader.TYPE: addon.type, patternloader.READ_ONLY: addon.isReadOnly()}
	patternloader._recordScriptAttributes(addonObj, addon)
	addonObj['owner'] = addon.owner.username
	accessList = []		
	addonObj['acl'] = acls.recordAccessList(addon.acl,accessList)
	return addonObj

def exportAddOns(dest):	
	addonsDir = os.path.join(dest, constants.ADDON_DIR)
	print "Exporting Add-Ons to directory: %s\n" % (addonsDir)
	if os.path.exists(addonsDir) :
		print "Destination %s exists. Please remove the directory before export. Skipping" % (addonsDir)
		print "-----------------\n"
		return 2
	patternloader._createDir(addonsDir)	
	i = 1
	for addon in deployer.addons :
		try:
			print "[%d] Exporting Add-On: %s ..." %(i,addon.name) 
			addonDir = os.path.join(addonsDir, str(i))
			patternloader._createDir(addonDir)
			addonObj = _recordAddonAttributes(addon)
					
			if addon.filename :
				addonArchiveFile = os.path.join(addonDir, addon.filename)
				addon.archive.get(addonArchiveFile)
				addonObj[patternloader.FILE] = addon.filename
				_verifyArchiveScript(addonDir, addon.filename)
			
			f = codecs.open(os.path.join(addonDir,JSON_FILE), 'wb', patternloader.UTF_8)
			try:
				f.write(str(patternloader._toJSON4J([addonObj]).serialize(True)))
			finally:
				f.close()	
			print "[%d] Add-On exported\n" % (i)
		except Exception, err:
			migration_exceptions.printExportException("AddOn",err,i)
		i = i + 1
	print "Export of Add-Ons completed"
	print "-----------------\n"

def importAddOns(dest,options=None):
	addonsDir = os.path.join(dest, constants.ADDON_DIR)
	print "Importing Add-Ons from directory: %s\n" % (addonsDir)
	if not os.path.exists(addonsDir) :
		print "Destination %s doesn't exists. Please export this artifact first. Exiting" % (addonsDir)
		print "-----------------\n"
		return 2
	i = 1
	for dir in os.listdir(addonsDir) :
		try:
			addonDir = os.path.join(addonsDir,dir)
			f = codecs.open(os.path.join(addonDir,JSON_FILE), 'r', patternloader.UTF_8)
			addonPackage = patternloader._parseJSON(f.read())[0]		
			f.close()
			if addonPackage :
				name = addonPackage[patternloader.NAME]
				type = addonPackage[patternloader.TYPE]
				addons = filter(lambda addon: addon.name == name, deployer.addons[name])
				print "[%d] Importing addon Package : %s ..." % (i,name)
				if len(addons) == 0 :					
					addon = deployer.addons << {patternloader.NAME: name, patternloader.TYPE: type}
					addons.append(addon)
					file = addonPackage.get(patternloader.FILE, None)
					if not file is None:
						path = os.path.join(addonDir, file)
						addon.archive.set(path)
						
					if options is not None:
						acls.createScoAccessList(addonPackage['acl'], RESOURCE_TYPE, addon.id, options['scp_hostname'], constants.SCO_HTTPS_PORT, options['sco_user'], options['sco_password'])
#						acls.changeOwnership(RESOURCE_TYPE, addon.id, addonPackage['owner'],options['scp_hostname'], constants.SCO_HTTPS_PORT, options['sco_user'], options['sco_password'])
						db2.changeResourceOwner(RESOURCE_TYPE, addon.id, addon.owner.id, addonPackage['owner'])
					patternloader._loadScriptAttributes(addonPackage, addons[0])
					print "[%d] Add-On imported\n" % (i)					
				else :
					print "[%d] Add-On %s already exists. Skipping\n" % (i,name)
		except Exception, err:			
			migration_exceptions.printImportException("AddOn",err,i)
		i = i+1
	print "Import of Add-Ons completed"
	print "-----------------\n"
	return 0
		
def _verifyArchiveScript(addonDir, addonFile):
	try:
		addonArchivePath = os.path.join(addonDir,addonFile)
		print '\tVerifying addon package %s ...' % (addonArchivePath)
		if _findCbscriptInArchive(addonArchivePath):
			print "\t'Cbscript.json' not present in file."
			return 0
		tmpExctractDir = os.path.join(addonDir, "tmpDir")
		patternloader._createDir(tmpExctractDir)	
		if addonArchivePath.endswith(".zip"):
			return _updateZipFile(addonArchivePath, tmpExctractDir)
		elif addonArchivePath.endswith(".tar.gz") or addonArchivePath.endswith(".tgz"):			
			return _updateTgzFile(addonArchivePath, tmpExctractDir)			
	except Exception, err:
		traceback.print_exc()
		return 1

def _findCbscriptInArchive(archivePath):
	cbscript = "cbscript.json"
	if archivePath.endswith(".zip"):
		cmd = ["/usr/bin/unzip", "-t", archivePath, cbscript] 
	elif archivePath.endswith(".tar.gz") or archivePath.endswith(".tgz"):
		cmd = ["/bin/tar", "-tf", archivePath, cbscript]
	child = subprocess.Popen(cmd,stdout=subprocess.PIPE)
	output = child.communicate()[0]
	if child.returncode == 0:
		return 0
	else:
		return 1

def _updateZipFile(zipPath,extractDir):
	cbscript = "cbscript.json"
	cbscriptPath = os.path.join(extractDir,cbscript)
	_unzipSingleFile(zipPath, extractDir,cbscript)
	if _modifyCbscript(cbscriptPath) == 0 :
		if _updateZipSingleFile(zipPath,extractDir,cbscript) == 0:
			print "\t%s updated" % (zipPath)
		else:
			print "\tError while updating %s" (zipPath)
	_removeDir(extractDir)

def _updateTgzFile(tgzPath, extractDir):
	_unpackTgzFile(tgzPath, extractDir)
	cbscript = "cbscript.json"
	cbscriptPath = os.path.join(extractDir,cbscript)
	if _modifyCbscript(cbscriptPath) == 0 :
		if _compressTgzFile(tgzPath,extractDir) == 0:
			print "\t%s repacked" % (tgzPath)
		else:
			print "\tError while repacking %s" (tgzPath)	
	_removeDir(extractDir)
	

def _unzipSingleFile(zipPath, fileDirectory, fileName):
	origWD = os.getcwd()
	os.chdir(fileDirectory)
	cmd = ["/usr/bin/unzip", zipPath, fileName]
	child = subprocess.Popen(cmd,stdout=subprocess.PIPE)
	output = child.communicate()[0]
	os.chdir(origWD)
	if child.returncode == 0:
		return 0
	else:
		return 1

def _updateZipSingleFile(zipPath, fileDirectory, fileName):
	origWD = os.getcwd()
	os.chdir(fileDirectory)
	cmd = ["/usr/bin/zip", "-uv", zipPath, fileName]
	child = subprocess.Popen(cmd,stdout=subprocess.PIPE)
	output = child.communicate()[0]
	os.chdir(origWD)
	if child.returncode == 0:
		return 0
	else:
		return 1
	
def _modifyCbscript(cbscriptPath):
	f = codecs.open(cbscriptPath, "r", patternloader.UTF_8)
	cbscript = patternloader._parseJSON(f.read())[0]		
	f.close()
	if cbscript.has_key("hypervisortypes"):
		hypervisortypes = patternloader._parseJSON(cbscript['hypervisortypes'])
		if constants.OPENSTACK_HYPERVISORTYPE not in hypervisortypes:
			print "\tValue of attribute 'hypervisortypes' is missing hypervisor 'OpenStack'"
			print "\tTool is going to modify cbscript.json"						
			cbscript["hypervisortypes"] = re.sub('"]', '\",\"OpenStack\"]', cbscript['hypervisortypes'])
			f = codecs.open(cbscriptPath, "wb", patternloader.UTF_8)
			try:
				f.write(str(patternloader._toJSON4J([cbscript]).serialize(True)))
				print "\t'cbscript.json' modified - added 'OpenStack' to 'hypervisortypes' list"
				return 0
			finally:
				f.close()		
	else:
		print "\tPackage correct"
		return 1
	

def _unpackZipFile(zipPath, extractDir):
	try:
		f = open(zipPath, "rb")
		z = zipfile.ZipFile(f)
		for name in z.namelist():
			outfile = open(os.path.join(extractDir,name),'wb')
			outfile.write(z.read(name))
			outfile.close()
		return 0
	except Exception, err:
		return 1
		

def _unpackTgzFile(tzgPath, extractDir):
	try:
# Following code doesn't work due to bug in jython present in IWD cli. 
# tarfile.extract() method is throwing and exception which breaks extract operation
# Due to this i've used subprocess to use system tar		
#		tfile = tarfile.open(tzgPath, 'r:gz')
#		for name in tfile:
#			tfile.extract(name,extractDir)
#		return 0
		cmd = ["/bin/tar", "-zxf", tzgPath, "-C", extractDir]
		child = subprocess.Popen(cmd,stdout=subprocess.PIPE)
		output = child.communicate()[0]
		if child.returncode == 0:
			return 0
		else:
			return 1
	except Exception, err:
		return 1
	
def _compressArchive(archivePath, dirToCompress):
	if archivePath.endswith(".zip"):
		return _compressZipFile(archivePath, dirToCompress)
	elif archivePath.endswith(".tar.gz") or archivePath.endswith(".tgz"):
		return _compressTgzFile(archivePath, dirToCompress)

def _compressZipFile(zipPath, dirToCompress):
	try:
		zfile = zipfile.ZipFile(zipPath, "w")
		root_len = len(os.path.abspath(dirToCompress))
		for root, dirs, files in os.walk(dirToCompress):
			archive_root = os.path.abspath(root)[root_len:]
			for file in files:
				fullpath = os.path.join(root, file)
				archive_name = os.path.join(archive_root, file)
	  			zfile.write(fullpath, archive_name)
	  	return 0
	except Exception, err:
		return 1  
	finally:
		zfile.close()
		
def _compressTgzFile(tgzPath, dirToCompress):
	try:
		tfile = tarfile.open(tgzPath, "w:gz")
		for name in os.listdir(dirToCompress):
			tfile.add(os.path.join(dirToCompress,name),arcname=name)
		return 0
	except Exception, err:
		return 1
	finally:
		tfile.close()
		
def _removeDir(dir):
	cmd = ["rm", "-rf", dir]
	child = subprocess.Popen(cmd,stdout=subprocess.PIPE)
	output = child.communicate()[0]
	if child.returncode == 0:
		return 0
	else:
		return 1
	


