'''
COPYRIGHT LICENSE:  This utility is owned by IBM and is provided in source code form. 
You may copy and use (but not modify)  the utility  without payment to IBM for the 
purposes of helping you to migrate your data from an IBM Workload Deployer product to
another IBM product. Notwithstanding anything to the contrary,  IBM PROVIDES THE UTILITY
ON AN "AS IS" BASIS AND IBM DISCLAIMS ALL WARRANTIES, EXPRESS OR IMPLIED, INCLUDING,
BUT NOT LIMITED TO, ANY IMPLIED WARRANTIES OR CONDITIONS OF MERCHANTABILITY, SATISFACTORY
QUALITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE, AND ANY WARRANTY OR CONDITION OF
NON-INFRINGEMENT. IBM SHALL NOT BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL
OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE OR OPERATION OF THE UTLITY.  
IBM MAY PROVIDE LIMITED SUPPORTFOR THE UTLITY, BUT IBM HAS NO OBLIGATION TO PROVIDE
MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS OR MODIFICATIONS TO THE UTLITITY.
'''

import os
import deployer
import codecs
import traceback
from deployer import patternloader
from scomigrator import projects, constants, migration_exceptions

JSON_FILE = 'group.json'

NAME="name"
DESCRIPTION="description"

def exportUserGroups(dest):
	groupsDir = os.path.join(dest, constants.GROUPS_DIR)
	print "Exporting User Groups to directory: %s ...\n" % (groupsDir)
	if os.path.exists(groupsDir) :
		print "Destination %s exists. Please remove the directory before export. Skipping" % (groupsDir)
		print "-----------------\n"
		return 2
	patternloader._createDir(groupsDir)	
	i = 1
	for group in deployer.groups :
		try:
			print "[%d] Exporting User Group: %s ..." %(i,group.name) 
			groupDir = os.path.join(groupsDir, str(i))
			patternloader._createDir(groupDir)		
			groupObj = {NAME: group.name, DESCRIPTION: group.description, "users": []}
			for user in group.users:
				groupObj['users'].append(user.username)
			f = codecs.open(os.path.join(groupDir,JSON_FILE), 'wb', patternloader.UTF_8)
			try:
				f.write(str(patternloader._toJSON4J([groupObj]).serialize(True)))
			finally:
				f.close()	
			print "[%d] User Group exported!\n" % (i)
		except Exception, err:
			migration_exceptions.printExportException("User Group",err,i)
		i = i + 1
	print "Export of Users completed"	
	print "-----------------\n"
	
def importUserGroups(authObj,envIds, dest):
	groupsDir = os.path.join(dest, constants.GROUPS_DIR)
	print "Importing User Groups (Projects) from directory: %s\n" % (groupsDir)
	if not os.path.exists(groupsDir) :
		print "Destination %s doesn't exists. Please export this artifact first. Exiting" % (groupsDir)
		print "-----------------\n"
		return 2
	i = 1
	for dir in os.listdir(groupsDir) :
		try:
			groupDir = os.path.join(groupsDir,dir)
			f = codecs.open(os.path.join(groupDir,JSON_FILE), 'r', patternloader.UTF_8)
			groupObj = patternloader._parseJSON(f.read())[0]		
			f.close()
			if groupObj :
				name = groupObj[NAME]				
				print "[%d] Importing User Group (Project) : %s ..." % (i,name)
				if name != constants.IWD_3107_DEFAULT_GROUP:								
					project = projects.getProject(authObj,name)
					if project is None :		
						projectRestObj = {"project": {
							"description": groupObj[DESCRIPTION],
							"domain_id": envIds['domain_id'],
							"enabled": "true",
							"name": name
						}}
						project = projects.createProject(authObj, projectRestObj)
						if project is not None:
							print "[%d] User Group (Project) created." % (i)
							print "[%d] Adding existing Users to Project ..." % (i)
							for username in groupObj['users']:
								from scomigrator import users								
								roles_to_add = constants.SCO_USER_ROLES
								for role in roles_to_add:
									print "[%d] Adding User %s with Role %s ..." % (i, username, role)
									if projects.addRoleForUserToProject(authObj, project['id'], username, role) == 0:
										print "[%d] Role added." %(i)
									else:
										print "[%d] Error while adding a role!" % (i)
								
							print "[%d] User Group imported\n" % (i)
						else:
							print "[%d] Error. User has not been imported\n" %(i)
					else:
						print "[%d] Project %s already exists. Skipping\n" % (i,name)				
				else :
					print "[%d] User Group %s will be skipped.\n" % (i,constants.IWD_3107_DEFAULT_GROUP)
		except Exception, err:			
			migration_exceptions.printImportException("Project",err,i)
			
		i = i +1
		
	print "Import of User Group (Projects) completed"
	print "-----------------\n"
	return 0