'''
COPYRIGHT LICENSE:  This utility is owned by IBM and is provided in source code form. 
You may copy and use (but not modify)  the utility  without payment to IBM for the 
purposes of helping you to migrate your data from an IBM Workload Deployer product to
another IBM product. Notwithstanding anything to the contrary,  IBM PROVIDES THE UTILITY
ON AN "AS IS" BASIS AND IBM DISCLAIMS ALL WARRANTIES, EXPRESS OR IMPLIED, INCLUDING,
BUT NOT LIMITED TO, ANY IMPLIED WARRANTIES OR CONDITIONS OF MERCHANTABILITY, SATISFACTORY
QUALITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE, AND ANY WARRANTY OR CONDITION OF
NON-INFRINGEMENT. IBM SHALL NOT BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL
OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE OR OPERATION OF THE UTLITY.  
IBM MAY PROVIDE LIMITED SUPPORTFOR THE UTLITY, BUT IBM HAS NO OBLIGATION TO PROVIDE
MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS OR MODIFICATIONS TO THE UTLITITY.
'''

import os
import codecs
import deployer
import traceback
from deployer.messages import message
from deployer import utils, ziputils, patternloader
from scomigrator import constants, migration_exceptions

ptype_status = {'available' : 'avail' , 'accepted'  : 'accepted'}
JSON_FILE = 'ptype.json'

SHORTNAME = "shortname"
STATUS = "status"
VERSION = "version"
FILE = "file"

def exportPatternTypes(dest):	
	ptypesDir = os.path.join(dest, constants.PTYPE_DIR)
	print "Exporting Pattern Types info to directory: %s \n" % (ptypesDir)
	if os.path.exists(ptypesDir) :
		print "Destination %s exists. Please remove the directory before export. Skipping" % (ptypesDir)
		print "-----------------\n"
		return 2
	patternloader._createDir(ptypesDir)	
	i = 1
	for ptype in deployer.patterntypes.list({VERSION:"vrmf"}) :
		try:
			print "[%d] Exporting Pattern-type: %s ..." %(i,ptype.name) 
			ptypeDir = os.path.join(ptypesDir, ptype.shortname+"_"+ptype.version)
			patternloader._createDir(ptypeDir)		
			ptypeObj = {patternloader.NAME: ptype.name, SHORTNAME: ptype.shortname, STATUS: ptype.status, VERSION: ptype.version, FILE:""}						
			f = codecs.open(os.path.join(ptypeDir,JSON_FILE), 'wb', patternloader.UTF_8)
			try:
				f.write(str(patternloader._toJSON4J([ptypeObj]).serialize(True)))
			finally:
				f.close()	
			print "[%d] Pattern exported!\n" % (i)
		except Exception, err:
			migration_exceptions.printExportException("Pattern Type",err,i)
		i = i + 1
	print "Export of Pattern Types completed"	
	print "-----------------\n"

def importPatternTypes(dest):
	ptypesDir = os.path.join(dest, constants.PTYPE_DIR)
	print "Importing Pattern Types from directory: %s\n" % (ptypesDir)
	if not os.path.exists(ptypesDir) :
		print "Destination %s doesn't exists. Please export this artifact first. Exiting" % (ptypesDir)
		print "-----------------\n"
		return 2
	i = 1
	for dir in os.listdir(ptypesDir) :
		try:
			ptypeDir = os.path.join(ptypesDir,dir)
			f = codecs.open(os.path.join(ptypeDir,JSON_FILE), 'r', patternloader.UTF_8)
			ptype = patternloader._parseJSON(f.read())[0]
			f.close()
			if ptype:
				print "[%d] Importing Pattern Type: %s %s ..." % (i,ptype[patternloader.NAME], ptype[VERSION])
				if checkIfPtypeExistsInIWD(ptype) :
					print "[%d] Pattern type already exists. Skipping" % (i)
				else :
					file = ptype.get(FILE,None)
					if not file is None and file != "" :
						print '[%d] Using file: %s' % (i,file)					
						pType = deployer.patterntypes.create(file)
						print '[%d] Pattern Type imported' % (i)
						print '[%d] Accepting license ....' % (i)
						status = pType[0].acceptLicense()
						if status[STATUS] == ptype_status['accepted'] :
							print '[%d] License accepted' % (i)
							if ptype[STATUS] == ptype_status['available']:
								print '[%d] Enabling pattern ....' % (i)
								status = pType[0].enable()
								if status[STATUS] == ptype_status['available']:
									print '[%d] Pattern enabled\n' %(i)
								else:
									print '[%d] Error while enabling pattern type\n' % (i)
						else:
							print  '[%d] Error while accepting license\n' % (i)					
					else:
						print '[%d] Pattern Type file not specified in %s. Skipping import\n' % (i,JSON_FILE)			
		except Exception, err:			
			migration_exceptions.printImportException("Pattern Type",err,i)		
		i = i+1
	print 'Import of Pattern  Types completed'
	print "-----------------\n"
	return 0

def checkIfPtypeExistsInIWD(ptype):
	try:
		shortname = ptype[SHORTNAME]
		version = ptype[VERSION]
		p = deployer.patterntypes.get(shortname, version)
		print p		
	except:
		return False
	return True
			
