'''
COPYRIGHT LICENSE:  This utility is owned by IBM and is provided in source code form. 
You may copy and use (but not modify)  the utility  without payment to IBM for the 
purposes of helping you to migrate your data from an IBM Workload Deployer product to
another IBM product. Notwithstanding anything to the contrary,  IBM PROVIDES THE UTILITY
ON AN "AS IS" BASIS AND IBM DISCLAIMS ALL WARRANTIES, EXPRESS OR IMPLIED, INCLUDING,
BUT NOT LIMITED TO, ANY IMPLIED WARRANTIES OR CONDITIONS OF MERCHANTABILITY, SATISFACTORY
QUALITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE, AND ANY WARRANTY OR CONDITION OF
NON-INFRINGEMENT. IBM SHALL NOT BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL
OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE OR OPERATION OF THE UTLITY.  
IBM MAY PROVIDE LIMITED SUPPORTFOR THE UTLITY, BUT IBM HAS NO OBLIGATION TO PROVIDE
MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS OR MODIFICATIONS TO THE UTLITITY.
'''

import codecs
import os
import deployer
import traceback
from deployer.messages import message
from deployer import patternloader
from scomigrator import acls,rest, db2, constants, migration_exceptions

JSON_FILE = 'script.json'
RESOURCE_TYPE="scripts"

def _recordScriptAttributes(script):
	scriptObj={patternloader.NAME: script.name, patternloader.EXEC_MODE: script.execmode, patternloader.READ_ONLY: script.isReadOnly()}
	patternloader._recordScriptAttributes(scriptObj, script)
	scriptObj['owner'] = script.owner.username
	accessList = []		
	scriptObj['acl'] = acls.recordAccessList(script.acl,accessList)
	return scriptObj
		
def exportScriptPackages(dest):	
	scriptsDir = os.path.join(dest, constants.SCRIPT_PACKAGE_DIR)
	print "Exporting Script Packages to directory: %s ...\n" % (scriptsDir)
	if os.path.exists(scriptsDir) :
		print "Destination %s exists. Please remove the directory before export. Skipping" % (scriptsDir)
		print "-----------------\n"
		return 2
	patternloader._createDir(scriptsDir)	
	i = 1
	for script in deployer.scripts :
		try:
			print "[%d] Exporting Script Package: %s ..." %(i, script.name) 
			scriptDir = os.path.join(scriptsDir, str(i))
			patternloader._createDir(scriptDir)		
			scriptObj = _recordScriptAttributes(script)
					
			if script.filename :
				script.archive.get(os.path.join(scriptDir, script.filename))
				scriptObj[patternloader.FILE] = script.filename
			
			f = codecs.open(os.path.join(scriptDir,JSON_FILE), 'wb', patternloader.UTF_8)
			try:
				f.write(str(patternloader._toJSON4J([scriptObj]).serialize(True)))
			finally:
				f.close()	
			print "[%d] Script Package exported\n" % (i)
		except Exception, err:
			migration_exceptions.printExportException("Script Package",err,i)
		i = i + 1
	print "Export of Script Packages completed"
	print "-----------------\n"

def importScriptPackages(dest, options=None):
	scriptsDir = os.path.join(dest, constants.SCRIPT_PACKAGE_DIR)
	print "Importing Script Packages from directory: %s\n" % (scriptsDir)
	if not os.path.exists(scriptsDir) :
		print "Destination %s doesn't exists. Please export this artifact first. Exiting" % (scriptsDir)
		print "-----------------\n"
		return 2
	i = 1
	for dir in os.listdir(scriptsDir) :
		try:
			scriptDir = os.path.join(scriptsDir,dir)
			f = codecs.open(os.path.join(scriptDir,JSON_FILE), 'r', patternloader.UTF_8)
			scriptPackage = patternloader._parseJSON(f.read())[0]		
			f.close()
			if scriptPackage :
				name = scriptPackage[patternloader.NAME]
				execmode = scriptPackage.get(patternloader.EXEC_MODE, None)
				scripts = filter(lambda script: script.name == name, deployer.scripts[name])
				print "[%d] Importing Script Package : %s ..." % (i,name)
				if len(scripts) == 0 :					
					script = deployer.scripts << {patternloader.NAME: name}
					scripts.append(script)
					file = scriptPackage.get(patternloader.FILE, None)
					if not file is None:
						path = os.path.join(scriptDir, file)
						script.archive.set(path)
				
					if execmode != None and scripts[0].execmode != execmode:
						scripts[0].execmode = execmode
					
					if options is not None:
						acls.createScoAccessList(scriptPackage['acl'], RESOURCE_TYPE, script.id, options['scp_hostname'], constants.SCO_HTTPS_PORT, options['sco_user'], options['sco_password'])
#						acls.changeOwnership(RESOURCE_TYPE, script.id, scriptPackage['owner'],options['scp_hostname'], constants.SCO_HTTPS_PORT, options['sco_user'], options['sco_password'])
					#	acls.changeOwnerRights(RESOURCE_TYPE, script.id, script.owner.id, scriptPackage['owner'],  options['db2_hostname'],  options['db2_user'])
						db2.changeResourceOwner(RESOURCE_TYPE, script.id, script.owner.id, scriptPackage['owner'])
					patternloader._loadScriptAttributes(scriptPackage, scripts[0])					
					print "[%d] Import completed\n" % (i)					
				else :
					print "[%d] Script package %s already exists. Skipping\n" % (i,name)
		except Exception, err:			
			migration_exceptions.printImportException("Script Package",err,i)
		i = i+1
	print "Import of ScriptPackages completed"
	print "-----------------\n"		
	return 0
