'''
COPYRIGHT LICENSE:  This utility is owned by IBM and is provided in source code form. 
You may copy and use (but not modify)  the utility  without payment to IBM for the 
purposes of helping you to migrate your data from an IBM Workload Deployer product to
another IBM product. Notwithstanding anything to the contrary,  IBM PROVIDES THE UTILITY
ON AN "AS IS" BASIS AND IBM DISCLAIMS ALL WARRANTIES, EXPRESS OR IMPLIED, INCLUDING,
BUT NOT LIMITED TO, ANY IMPLIED WARRANTIES OR CONDITIONS OF MERCHANTABILITY, SATISFACTORY
QUALITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE, AND ANY WARRANTY OR CONDITION OF
NON-INFRINGEMENT. IBM SHALL NOT BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL
OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE OR OPERATION OF THE UTLITY.  
IBM MAY PROVIDE LIMITED SUPPORTFOR THE UTLITY, BUT IBM HAS NO OBLIGATION TO PROVIDE
MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS OR MODIFICATIONS TO THE UTLITITY.
'''

import sys
import os
import deployer
import codecs
import traceback
from deployer import patternloader
from scomigrator import rest, projects, constants, migration_exceptions

JSON_FILE = 'user.json'
RESOURCE_TYPE = "users"

USERNAME="username"
EMAIL="email"
FULLNAME="fullname"
RESOURCE_TYPE = "users"

def createUser(authObj, userRestObj):	
	response = rest.createKeystoneResource(authObj, RESOURCE_TYPE, userRestObj)
	if response['status'] in rest.ERROR_STATUSES:
		print "Error while creating a user: %s. Response status is %s" % (userRestObj['user']['name'],response)	
	else:
		return response['response']['user']

def getUser(authObj, username):
	return rest.getKeystoneResource(authObj, RESOURCE_TYPE, username)		

def exportUsers(dest):
	usersDir = os.path.join(dest, constants.USERS_DIR)
	print "Exporting Users to directory: %s ...\n" % (usersDir)
	if os.path.exists(usersDir) :
		print "Destination %s exists. Please remove the directory before export. Skipping" % (usersDir)
		print "-----------------\n"
		return 2
	patternloader._createDir(usersDir)	
	i = 1
	for user in deployer.users :
		try:
			print "[%d] Exporting User: %s ..." %(i,user.username) 
			userDir = os.path.join(usersDir, str(i))
			patternloader._createDir(userDir)		
			userObj = {USERNAME: user.username, EMAIL: user.email, FULLNAME: user.fullname}						
			f = codecs.open(os.path.join(userDir,JSON_FILE), 'wb', patternloader.UTF_8)
			try:
				f.write(str(patternloader._toJSON4J([userObj]).serialize(True)))
			finally:
				f.close()	
			print "[%d] User exported!\n" % (i)
		except Exception, err:
			migration_exceptions.printExportException("User",err,i)
		i = i + 1
	print "Export of Users completed"	
	print "-----------------\n"

def exportUsersAndGroups(dest):
	exportUsers(dest)
	groups.exportUserGroups(dest)

def importUsersAndGroups(authObj, envIds, dest):
	importUsers(authObj,envIds, dest)
	groups.importUserGroups(authObj, envIds, dest)
	return 0

def importUsers(authObj,envIds, dest):		
	usersDir = os.path.join(dest, constants.USERS_DIR)
	print "Importing Users from directory: %s\n" % (usersDir)
	if not os.path.exists(usersDir) :
		print "Destination %s doesn't exists. Please export this artifact first. Exiting" % (usersDir)
		print "-----------------\n"
		return 2
	i = 1
	for dir in os.listdir(usersDir) :
		try:
			userDir = os.path.join(usersDir,dir)
			f = codecs.open(os.path.join(userDir,JSON_FILE), 'r', patternloader.UTF_8)
			userObj = patternloader._parseJSON(f.read())[0]		
			f.close()
			if userObj :
				name = userObj[USERNAME]
				print "[%d] Importing User : %s ..." % (i,name)				
				user = getUser(authObj, name)
				if user is None :
					if name == constants.IWD_3107_SUPERUSER :
						print "[%d] User %s will not be imported. Skipping\n" % (i,constants.IWD_3107_SUPERUSER)
						i = i + 1
						continue
					password = constants.SCO_USER_DEFAULT_PASSWORD
					parameters = { "user": {
						"default_project_id": envIds['project_id'],
						"tenantId" : envIds['project_id'],
						"description": "",
						"domain_id": envIds['domain_id'],
						"email": userObj[EMAIL],
						"enabled": "true",
						"name": name,
						"password": password
					}}					
					user = createUser(authObj, parameters)
					if user is not None:
						print "[%d] User created" % (i)
						roles_to_add = constants.SCO_USER_ROLES						
						for role in roles_to_add:
							print "[%d] Adding User %s with Role %s to default project.." % (i, name, role)
							if projects.addRoleForUserToProject(authObj, envIds['project_id'], name, role) == 0:
								print "[%d] Role added." %(i)
							else:
								print "[%d] Error while adding a role!" % (i)
						print "[%d] User imported\n" % (i)
					else:
						print "[%d] Error. User has not been imported\n" %(i)
				else:
					print "[%d] User %s already exists. Skipping\n" % (i,name)
		except Exception, err:
			migration_exceptions.printImportException("User",err,i)
		i = i +1
		
	print "Import of Users completed"
	print "-----------------\n"
	return 0



