'''
COPYRIGHT LICENSE:  This utility is owned by IBM and is provided in source code form. 
You may copy and use (but not modify)  the utility  without payment to IBM for the 
purposes of helping you to migrate your data from an IBM Workload Deployer product to
another IBM product. Notwithstanding anything to the contrary,  IBM PROVIDES THE UTILITY
ON AN "AS IS" BASIS AND IBM DISCLAIMS ALL WARRANTIES, EXPRESS OR IMPLIED, INCLUDING,
BUT NOT LIMITED TO, ANY IMPLIED WARRANTIES OR CONDITIONS OF MERCHANTABILITY, SATISFACTORY
QUALITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE, AND ANY WARRANTY OR CONDITION OF
NON-INFRINGEMENT. IBM SHALL NOT BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL
OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE OR OPERATION OF THE UTLITY.  
IBM MAY PROVIDE LIMITED SUPPORTFOR THE UTLITY, BUT IBM HAS NO OBLIGATION TO PROVIDE
MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS OR MODIFICATIONS TO THE UTLITITY.
'''

import os
import codecs
import deployer
import traceback
from deployer.messages import message
from deployer import patternloader

from scomigrator import acls, rest, constants, db2, migration_exceptions

JSON_FILE = "vapp.json"
MODEL_FILE = "model.json"
RESOURCE_TYPE="applicationPatterns"

def _recordVappAttributes(vapp):
	vappObj={"app_name": vapp.app_name}
	vappObj['creator'] = vapp.creator
	accessList = []		
	vappObj['acl'] = acls.recordAccessList(vapp.acl,accessList)
	return vappObj

def exportVirtualApplicationPatterns(dest):	
	vappsDir = os.path.join(dest, constants.VAPP_DIR)
	print "Exporting Virtual Application Patterns to: %s\n" % (vappsDir)
	if os.path.exists(vappsDir) :
		print "Destination %s exists. Please remove the directory before export. Skipping" % (vappsDir)
		print "-----------------\n"
		return 2
	patternloader._createDir(vappsDir)
	i=1
	for vapp in deployer.applications :	
		try:	
			vappDir = os.path.join(vappsDir, str(i))
			print "[%d] Exporting: %s to %s ..." % (i,vapp.app_name,vappDir)		
			patternloader._createDir(vappDir)		
			vapp.download(os.path.join(vappDir,MODEL_FILE))
			dirList = os.listdir(vappDir)
			if len(dirList):
				vappObj = _recordVappAttributes(vapp)
				f = codecs.open(os.path.join(vappDir,JSON_FILE), 'wb', patternloader.UTF_8)
				try:
					f.write(str(patternloader._toJSON4J([vappObj]).serialize(True)))
				finally:
					f.close()				
				print "[%d] Virtual Application Pattern exported\n" % (i)
			else:
				print "[%d] Error while exporting Virtuall Application Pattern. Not exported" % (i)
		except Exception, err:
			migration_exceptions.printExportException("Virtual Application Pattern",err,i)
		i = i+ 1
	print "Virtual Application Patterns export completed"
	print "-----------------\n"

def importVirtualApplicationPatterns(dest, options):	
	vappsDir = os.path.join(dest, constants.VAPP_DIR)
	print "Importing Virtual Application Patterns from directory: %s\n" %(vappsDir)
	if not os.path.exists(vappsDir) :
		print "Destination %s doesn't exists. Please export this artifact first. Exiting" % (vappsDir)
		print "-----------------\n"
		return 2
	i = 1
	for dir in os.listdir(vappsDir) :
		try:
			vappDir = os.path.join(vappsDir,dir)
			vappPath = os.path.join(vappDir,MODEL_FILE)
			f = codecs.open(os.path.join(vappDir,JSON_FILE), 'r', patternloader.UTF_8)
			vappPattern = patternloader._parseJSON(f.read())[0]
			f.close()		
			print "[%d] Importing: %s ..." % (i,vappPattern['app_name'])	
			vappList = deployer.applications.list({'app_name':vappPattern['app_name']})
			if len(vappList) > 0 :
				print "[%d] Pattern already exists. Skipping\n" % (i)
			else:				
				deployer.applications.create(vappPath)	
				vappList = deployer.applications.list({'app_name':vappPattern['app_name']})
				if len(vappList) :
					vapp = vappList[0]
					print "[%d] Creating access list for a resource ... " %(i)
					acls.createScoAccessList(vappPattern['acl'], RESOURCE_TYPE, vapp.app_id, options['scp_hostname'], constants.SCO_HTTPS_PORT, options['sco_user'], options['sco_password'])
				#	acls.changeOwnership(RESOURCE_TYPE, vapp.app_id, vappPattern['creator'],options['scp_hostname'], constants.SCO_HTTPS_PORT, options['sco_user'], options['sco_password'])
					print "[%d] Access List modified" %(i)
					print "[%d] Pattern imported\n"  % (i)
				else:
					print "[%d] Error while importing a pattern. Pattern not imported" %(i)
		except Exception, err:
			migration_exceptions.printImportException("Virtual Application Pattern",err,i)
		i = i+1
	print "Virtual Applications import completed"
	print "-----------------\n"
	return 0


