'''
COPYRIGHT LICENSE:  This utility is owned by IBM and is provided in source code form. 
You may copy and use (but not modify)  the utility  without payment to IBM for the 
purposes of helping you to migrate your data from an IBM Workload Deployer product to
another IBM product. Notwithstanding anything to the contrary,  IBM PROVIDES THE UTILITY
ON AN "AS IS" BASIS AND IBM DISCLAIMS ALL WARRANTIES, EXPRESS OR IMPLIED, INCLUDING,
BUT NOT LIMITED TO, ANY IMPLIED WARRANTIES OR CONDITIONS OF MERCHANTABILITY, SATISFACTORY
QUALITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE, AND ANY WARRANTY OR CONDITION OF
NON-INFRINGEMENT. IBM SHALL NOT BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL
OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE OR OPERATION OF THE UTLITY.  
IBM MAY PROVIDE LIMITED SUPPORTFOR THE UTLITY, BUT IBM HAS NO OBLIGATION TO PROVIDE
MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS OR MODIFICATIONS TO THE UTLITITY.
'''

import os
import deployer
from deployer import utils, ziputils, patternloader
import time
import base64
import httplib
import codecs
from scomigrator import acls,rest,constants, db2, migration_exceptions

JSON_FILE = "virtimg.json"
RESOURCE_TYPE="templates"
port = 443

def _recordImageAttributes(image):
	imageObj={patternloader.NAME: image.name, "licenseaccepted" : image.licenseaccepted, "currentstatus" : image.currentstatus }
	imageObj['owner'] = image.owner.username
	accessList = []		
	imageObj['acl'] = acls.recordAccessList(image.acl,accessList)
	return imageObj

def exportVirtualImages(dest):	
	virtImgsDir = os.path.join(dest, constants.VIRT_DIR)
	print "Exporting Virtual Images to: %s" % (virtImgsDir)
	print "This operation may take a while, so be patient\n"
	if os.path.exists(virtImgsDir) :
		print "Destination %s exists. Please remove the directory before export. Skipping" % (virtImgsDir)
		print "-----------------\n"
		return 2
	patternloader._createDir(virtImgsDir)
	i = 1
	for image in deployer.virtualimages :
		try:
			virtImgDir = os.path.join(virtImgsDir, str(i))
			print '[%d] Exporting Virtual Image %s to %s ...' % (i,image.name, virtImgDir)
			patternloader._createDir(virtImgDir)
	#		image.export({
	#          'host': export_data['host'],
	#          'path': virtImgsDir ,
	#		  'userid': export_data['userid'],
	#		  'password': export_data['password']
	#		})
			imgStatus = image.currentstatus
			imageObj = _recordImageAttributes(image)
			image.export(virtImgDir)				
			time.sleep(5)
			# Loop was developed for asynchronuous export.
			# If we export virt image for the same machine export is run synchronously.
			# Below loop will be run just one.
			counter = 0		
			while counter < 1000 :			
				curImg = deployer.virtualimages[image.name][0]
				status = curImg.currentstatus
				if status == constants.VIRT_IMAGE_STATUS['exporting'] :
					counter = counter + 1
					print '\tStatus %s. Image not yet exported. Sleeping for 30 sec. Counter = %d' % (curImg.currentstatus_text,counter)
					time.sleep(30)
				elif status == imgStatus :
					imgName = os.listdir(virtImgDir)[0]
					if imgName:
						imageObj['filename'] = imgName
						f = codecs.open(os.path.join(virtImgDir,JSON_FILE), 'wb', patternloader.UTF_8)
						try:
							f.write(str(patternloader._toJSON4J([imageObj]).serialize(True)))
						finally:
							f.close()	
						print '[%d] Image has been exported\n' % (i)
					else:
						print '[%d] Error while exporting Virtual Image. Exported OVA does not exists.' %(i)
					break
				else:
					print "[%d] Error while exporting Virtual Image. Export stopped. Image status : %s\n" % (i,curImg.currentstatus_text)
					print "[%d] Message: %s\n" % (i, curImg.currentmessage)					
		except Exception, err:			
			migration_exceptions.printExportException("Virtual Image",err,i)
			
		i = i + 1
	print "Export of Virtual Images completed"
	print "-----------------\n"

def importVirtualImages(dest,options=None):
	virtImgsDir = os.path.join(dest, constants.VIRT_DIR)
	print "Importing Virtual Images from directory %s\n" % (virtImgsDir)
	if not os.path.exists(virtImgsDir) :
		print "Destination %s doesn't exists. Please export this artifact first. Exiting" % (virtImgsDir)
		print "-----------------\n"
		return 2
	i = 1
	for dir in os.listdir(virtImgsDir) :
		try:
			virtImgDir = os.path.join(virtImgsDir,dir)
			propertyFile = os.path.join(virtImgDir,JSON_FILE)
			if not os.path.exists(propertyFile):
				print "[%d] Json file %s for image doesn't exists. Skipping import\n" % (i, propertyFile)
				i = i+1
				continue
			f = codecs.open(propertyFile, 'r', patternloader.UTF_8)
			virtImage = patternloader._parseJSON(f.read())[0]		
			f.close()
			imgName = virtImage['filename']
			imgStatus = virtImage['currentstatus']
			if imgName:
				print "[%d] Importing: %s ..." % (i,imgName)
				virtualImages = filter(lambda virtualImage: virtualImage.name == virtImage['name'], deployer.virtualimages)				
				if len(virtualImages) > 0 :
					print "[%d] Virtual Image already exists. Skipping\n" % (i)
					i = i + 1
					continue
				image = deployer.virtualimages.create(os.path.join(virtImgDir,imgName))			
				time.sleep(5)
				counter = 0
				while counter < 1000 :
					virtImgs = filter(lambda virtimg: virtimg.id == image.id, deployer.virtualimages)
					status = virtImgs[0].currentstatus
					if status in [constants.VIRT_IMAGE_STATUS['processing'], constants.VIRT_IMAGE_STATUS['queued']]:
						counter = counter + 1
						print "\tStatus %s. Image not yet imported. Sleeping for 30 sec" % (virtImgs[0].currentstatus_text)
						time.sleep(30)
					elif status == constants.VIRT_IMAGE_STATUS['failed']:
						print "[%d] Import stopped!. Image status : %s" %(i,virtImgs[0].currentstatus_text)
						print "[%d] Message: %s\n" % (i, virtImgs[0].currentmessage)
						break
					else:				
						print "[%d] Image has been imported" %(i)
						if virtImage['licenseaccepted'] == "T":
							print "[%d] Accepting License ..." %(i)
							virtImgs[0].acceptLicense()
							if virtImgs[0].currentstatus == constants.VIRT_IMAGE_STATUS['read_only'] :
								print "[%d] License accepted\n" %(i)
							else:
								print "[%d] Error while accepting the license\n" %(i)
						if options is not None:
							if imgStatus == constants.VIRT_IMAGE_STATUS['draft']:
								print "[%d] Changing state to 'DRAFT'" % (i)
								_changeImageStatus(image.id, imgStatus, options['scp_hostname'], constants.SCO_HTTPS_PORT, options['sco_user'], options['sco_password'])
								
							print "[%d] Creating access list for a resource ... " %(i)
							acls.createScoAccessList(virtImage['acl'], RESOURCE_TYPE, image.id, options['scp_hostname'], constants.SCO_HTTPS_PORT, options['sco_user'], options['sco_password'])
			#				acls.changeOwnership(RESOURCE_TYPE, image.id, virtImage['owner'],options['scp_hostname'], constants.SCO_HTTPS_PORT, options['sco_user'], options['sco_password'])
							db2.changeResourceOwner(RESOURCE_TYPE, image.id, image.owner.id, virtImage['owner'])
							print "[%d] Access List modified" %(i)					
						break
		except Exception, err:			
			migration_exceptions.printImportException("Virtual Image",err,i)
					
		i = i+1
	print "Import of Virtual Images completed"
	print "Please proceed to VIL UI, in order to check out imported images"
	print "-----------------\n"
	return 0

def _changeImageStatus(imgId, imgStatus, host,port, username, password):
	url = "/resources/"+RESOURCE_TYPE+"/"+ str(imgId)
	body = { "currentstatus" : imgStatus}
	response = rest.iwdRestRequest(host, port, username, password, "PUT", url, str(body))
	if response['status'] == rest.STATUS_GET_SUCCESS :
		print "\tStatus changed to %s" % (imgStatus)
	else:
		print "\tError while changing status. Response: %s %s" % (response['status'], response['response'])
					
	
def checkOutVirtualImages(host, repoName, datastoreName, username, password):
	print "Checking Out Virtual Images on VIL to Operational Repository %s ... \n" % (repoName)
	isError = 0
	repoUID = getVilRepositoryUID(host,repoName,datastoreName, username, password)
	if repoUID != None :
		imageIdList = getVilImagesUIDs(host, username, password)
		if imageIdList != None :
			url = "/ImageLibrary/ImageLibrary/referenceImage/checkout"
			body = {'instanceRepositoryUUID':str(repoUID),'referenceImageUUID':'','name':'',}
			i = 1			
			for image in imageIdList:
				name = getImageNameByUID(host,image, username, password)
				print "[%d] Checking Out image: %s" % (i, name)				
				body['name'] = str(name)[:31]
				body['referenceImageUUID'] = str(image)				
				response = rest.iwdRestRequest(host,constants.SCO_HTTPS_PORT, username, password, "POST", url, str(body))
				if response['status'] == rest.STATUS_GET_SUCCESS:
					task = patternloader._parseJSON(response['response'])
					print "[%d] Check Out has been started for the image. Task id for this operation is %s\n" % (i,task['task_uuid'])
				else:
					print "[%d] Error while checking out image. Response status: %d" % (i, response['status'])
					print "[%d] Exception: %s\n" % (i,response['response'])
					isError = 1
				i = i+1
		else:
			print "Cannot find virtual images to check out on VIL\n"
			isError = 1
	else :
		print "Cannot localize Operational Repository  %s with datastore %s on VIL\n" % (repoName,datastoreName)
		isError = 1
			
	if isError:
		print "Check Out of virtual images run with errors"
	else:
		print "Check Out of virtual images scheduled"
		print "To ensure tasks are completed successfully, login to VIL UI and check the operation progress."
	print "-----------------\n"
	return isError

def getVilRepositoryUID(host, repoName, datastoreName, username, password):
	url ="/ImageLibrary/ImageLibrary/instanceRepository?function=listRepositories"
	response = rest.iwdRestRequest(host,constants.SCO_HTTPS_PORT, username, password, "GET", url, None)
	if response['status'] == rest.STATUS_GET_SUCCESS:	
		repoIds = patternloader._parseJSON(response['response'])		
		uid = None
		for id in repoIds :
			uri = "/ImageLibrary/ImageLibrary/instanceRepository?function=describeRepository&repositoryUUID=" + id
			resp = rest.iwdRestRequest(host,constants.SCO_HTTPS_PORT, username, password, "GET", uri, None)
			if resp['status'] == rest.STATUS_GET_SUCCESS :
				repo = patternloader._parseJSON(resp['response'])
				if repo['name'] == datastoreName and repo['domainName'] == repoName :
					uid = id
			if uid != None :
				return uid
		return None
	else:
		return None

def getVilImagesUIDs(host, username, password):
	imageList = None
	url = "/ImageLibrary/ImageLibrary/referenceImage?function=listImages"	
	response = rest.iwdRestRequest(host,constants.SCO_HTTPS_PORT, username, password, "GET", url, None)
	if response['status'] == rest.STATUS_GET_SUCCESS:
		imageList = patternloader._parseJSON(response['response'])
	return imageList
	
def getImageNameByUID(host, imageId, username, password):
	imageName = ""
	url = "https://"  + host + "/ImageLibrary/ImageLibrary/referenceImage?function=describeImage&referenceImageUUID=" + imageId
	response = rest.iwdRestRequest(host,constants.SCO_HTTPS_PORT, username, password, "GET", url, None)
	if response['status'] == rest.STATUS_GET_SUCCESS:
		image = patternloader._parseJSON(response['response'])
		imageName = image['name']
	return imageName
	