'''
COPYRIGHT LICENSE:  This utility is owned by IBM and is provided in source code form. 
You may copy and use (but not modify)  the utility  without payment to IBM for the 
purposes of helping you to migrate your data from an IBM Workload Deployer product to
another IBM product. Notwithstanding anything to the contrary,  IBM PROVIDES THE UTILITY
ON AN "AS IS" BASIS AND IBM DISCLAIMS ALL WARRANTIES, EXPRESS OR IMPLIED, INCLUDING,
BUT NOT LIMITED TO, ANY IMPLIED WARRANTIES OR CONDITIONS OF MERCHANTABILITY, SATISFACTORY
QUALITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE, AND ANY WARRANTY OR CONDITION OF
NON-INFRINGEMENT. IBM SHALL NOT BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL
OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE OR OPERATION OF THE UTLITY.  
IBM MAY PROVIDE LIMITED SUPPORTFOR THE UTLITY, BUT IBM HAS NO OBLIGATION TO PROVIDE
MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS OR MODIFICATIONS TO THE UTLITITY.
'''

import sys
import os
import deployer
from deployer.messages import message
from deployer import utils, ziputils, patternloader
import time
import codecs
from scomigrator import acls, rest, constants, db2, migration_exceptions

RESOURCE_TYPE="patterns"
JSON_FILE="pattern.json"
passwords = True
download = None
downloadAll = False

def _recordVsysAttributes(pattern):
	vsysObj={patternloader.NAME: pattern.name}
	vsysObj['owner'] = pattern.owner.username
	accessList = []		
	vsysObj['acl'] = acls.recordAccessList(pattern.acl,accessList)	
	return vsysObj

def exportVirtualSystemPatterns(dest):	
	vsyssDir = os.path.join(dest, constants.VSYS_DIR)
	print "Exporting Virtual System Patterns to %s \n" % (vsyssDir)
	if os.path.exists(vsyssDir) :
		print "Destination %s exists. Please remove the directory before export. Skipping" % (vsyssDir)
		print "-----------------\n"
		return 2
	patternloader._createDir(vsyssDir)
	i=1
	for pattern in deployer.patterns :		
		vsysDir = os.path.join(vsyssDir, str(i))
		print "[%d] Exporting Virtual System %s to %s" % (i,pattern.name,vsysDir)
		try:
			patternloader._createDir(vsysDir)		
			pattern.save(vsysDir, includePasswords=passwords, download=download, downloadAll = downloadAll)
			dirList = os.listdir(vsysDir)
			if len(dirList):
				vsysObj = _recordVsysAttributes(pattern)
				f = codecs.open(os.path.join(vsysDir,JSON_FILE), 'wb', patternloader.UTF_8)
				try:
					f.write(str(patternloader._toJSON4J([vsysObj]).serialize(True)))
				finally:
					f.close()
				print "[%d] Virtual System Pattern exported\n" % (i)
		except Exception, err:
			migration_exceptions.printExportException("Virtual System Pattern",err,i)
			
		i = i+ 1
	print "Export of Virtual System Patterns completed"
	print "-----------------\n"

def importVirtualSystemPatterns(dest,options):	
	vsyssDir = os.path.join(dest, constants.VSYS_DIR)
	print "Importing Virtual System Patterns from directory: %s\n" % (vsyssDir)
	if not os.path.exists(vsyssDir) :
		print "Destination %s doesn't exists. Please export this artifact first. Exiting" % (vsyssDir)
		print "-----------------\n"
		return 2
	i = 1
	for dir in os.listdir(vsyssDir) :
		try:
			vsysDir = os.path.join(vsyssDir,dir)
			f = codecs.open(os.path.join(vsysDir,JSON_FILE), 'r', patternloader.UTF_8)
			vsysPattern = patternloader._parseJSON(f.read())[0]		
			f.close()
			if vsysPattern:			
				print "[%d] Importing Virtual System %s ..." % (i,vsysPattern['name'])
				vsysPatterns = filter(lambda vsysPat: vsysPat.name == vsysPattern['name'], deployer.patterns)				
				if len(vsysPatterns) > 0 :
					print "[%d] Pattern already exists. Skipping\n" % (i)				
				else:				
					deployer.patterns.load(vsysDir)					
					vsysPatterns = filter(lambda vsysPat: vsysPat.name == vsysPattern['name'], deployer.patterns)
					if len(vsysPatterns) == 0 :
						print "[%d] Error. Pattern not available after import.\n" % (i)
						i = i + 1
						continue				
					pattern = vsysPatterns[0]
					if pattern:
						if options is not None:
							print "[%d] Creating access list for a resource ... " %(i)
							acls.createScoAccessList(vsysPattern['acl'], RESOURCE_TYPE, pattern.id, options['scp_hostname'], constants.SCO_HTTPS_PORT, options['sco_user'], options['sco_password'])
							#acls.changeOwnership(RESOURCE_TYPE, pattern.id, vsysPattern['owner'],options['scp_hostname'], constants.SCO_HTTPS_PORT, options['sco_user'], options['sco_password'])
							db2.changeResourceOwner(RESOURCE_TYPE, pattern.id, pattern.owner.id, vsysPattern['owner'])
							print "[%d] Access List modified" %(i)
						print "[%d] Pattern imported\n" % (i)
					else:
						print "[%d] Error while importing a pattern. Not imported" %(i)
		except Exception, err:
			migration_exceptions.printImportException("Virtual System Pattern",err,i)
			
		i = i+1
	print "Import of Virtual System Patterns completed"
	print "-----------------\n"
	return 0
		


